#-*- encoding=utf-8 -*-
import os
from datetime import datetime, timedelta

from django.core.management.base import BaseCommand
from django.conf import settings


class Command(BaseCommand):
    help = 'Purge xlsx and csvs'

    def handle(self, *args, **options):
        directories = ['xlsx', 'csvs']
        for directory in directories:
            path = os.path.join(settings.MEDIA_ROOT, directory)
            files = os.listdir(path)
            purge_date = datetime.now() - timedelta(days=7)
            for file in files:
                file = os.path.join(path, file)
                file_date = datetime.fromtimestamp(os.path.getmtime(file))
                print file_date
                if file_date < purge_date:
                    os.remove(file)



#-*- encoding=utf-8 -*-
from datetime import datetime

from django.core.management.base import BaseCommand

from queryset_reporter.models import Queryset
from queryset_reporter.core import Reporter


class Command(BaseCommand):
    help = 'Automatic report generation'

    def handle(self, *args, **options):
        querysets = Queryset.objects.filter(automatic_generation=True)
        for queryset in querysets:
            reporter = Reporter(queryset, None)
            xlsx = reporter.render_xlsx()
            queryset.last_automatic_generation_at = datetime.now()
            queryset.last_automatic_generation_xlsx = xlsx
            queryset.save()



# -*- encoding: utf-8 -*-

'''
mapping of lookups and fields.
'''

from django.utils.translation import ugettext_lazy as _

LOOKUPS = (
    ('exact', _(u'Termo exato')),
    ('iexact', _(u'Termo exato (case-insensitivo)')),
    ('contains', _(u'Contém o termo')),
    ('icontains', _(u'Contém o termo (case-insensitivo)')),
    ('in', _(u'Termo está na lista')),
    ('gt', _(u'Maior que')),
    ('gte', _(u'Maior ou igual que')),
    ('lt', _(u'Menor que')),
    ('lte', _(u'Menor ou igual que')),
    ('startswith', _(u'Começa com')),
    ('istartswith', _(u'Começa com (case-insensitivo)')),
    ('endswith', _(u'Termina com')),
    ('iendswith', _(u'Termina com (case-insensitivo)')),
    ('range', _(u'Faixa/período')),
    ('year', _(u'Ano específico')),
    ('month', _(u'Mês específico')),
    ('day', _(u'Dia específico.')),
    ('isnull', _(u'É nulo?')),
    ('search', _(u'Busca textual')),  # mysql
    ('regex', _(u'Expressão regular')),
    ('iregex', _(u'Expressão regular (case-insensitivo)')),
)

_NUMERICAL_LOOKUP = {
    'exact': (1, 'numerical-field'),
    'iexact': (1, 'numerical-field'),
    'contains': None,
    'icontains': None,
    'in': (1, 'numerical-list'),
    'gt': (1, 'numerical-field'),
    'gte': (1, 'numerical-field'),
    'lt': (1, 'numerical-field'),
    'lte': (1, 'numerical-field'),
    'startswith': None,
    'istartswith': None,
    'endswith': None,
    'iendswith': None,
    'range': (2, 'numerical-field'),
    'year': None,
    'month': None,
    'day': None,
    'isnull': (1, 'boolean-field'),
    'search': None,
    'regex': None,
    'iregex': None,
}

_CHAR_LOOKUP = {
    'exact': (1, 'char-field'),
    'iexact': (1, 'char-field'),
    'contains': (1, 'char-field'),
    'icontains': (1, 'char-field'),
    'in': (1, 'char-list'),
    'gt': None,
    'gte': None,
    'lt': None,
    'lte': None,
    'startswith': (1, 'char-field'),
    'istartswith': (1, 'char-field'),
    'endswith': (1, 'char-field'),
    'iendswith': (1, 'char-field'),
    'range': (2, 'numerical-field'),
    'year': None,
    'month': None,
    'day': None,
    'isnull': (1, 'boolean-field'),
    'search': (1, 'char-field'),
    'regex': (1, 'char-field'),
    'iregex': (1, 'char-field'),
}

_DATETIME_LOOKUP = {
    'exact': (1, 'datetime-field'),
    'iexact': (1, 'datetime-field'),
    'contains': None,
    'icontains': None,
    'in': None,
    'gt': (1, 'datetime-field'),
    'gte': (1, 'datetime-field'),
    'lt': (1, 'datetime-field'),
    'lte': (1, 'datetime-field'),
    'startswith': None,
    'istartswith': None,
    'endswith': None,
    'iendswith': None,
    'range': (2, 'datetime-field'),
    'year': (1, 'numerical-field'),
    'month': (1, 'numerical-field'),
    'day': (1, 'numerical-field'),
    'isnull': (1, 'boolean-field'),
    'search': None,
    'regex': None,
    'iregex': None,
}

_BOOLEAN_LOOKUP = {
    'exact': (1, 'boolean-field'),
    'iexact': None,
    'contains': None,
    'icontains': None,
    'in': None,
    'gt': None,
    'gte': None,
    'lt': None,
    'lte': None,
    'startswith': None,
    'istartswith': None,
    'endswith': None,
    'iendswith': None,
    'range': None,
    'year': None,
    'month': None,
    'day': None,
    'isnull': (1, 'boolean-field'),
    'search': None,
    'regex': None,
    'iregex': None,
}

_RELATED_LOOKUP = {
    'exact': (1, 'numerical-field'),
    'iexact': None,
    'contains': None,
    'icontains': None,
    'in': (1, 'numerical-list'),
    'gt': None,
    'gte': None,
    'lt': None,
    'lte': None,
    'startswith': None,
    'istartswith': None,
    'endswith': None,
    'iendswith': None,
    'range': None,
    'year': None,
    'month': None,
    'day': None,
    'isnull': (1, 'boolean-field'),
    'search': None,
    'regex': None,
    'iregex': None,
}

LOOKUP_MAPPING = {
    # numericals
    'BigIntegerField': _NUMERICAL_LOOKUP,
    'IntegerField': _NUMERICAL_LOOKUP,
    'DecimalField': _NUMERICAL_LOOKUP,
    'FloatField': _NUMERICAL_LOOKUP,
    'PositiveIntegerField': _NUMERICAL_LOOKUP,
    'PositiveSmallIntegerField': _NUMERICAL_LOOKUP,
    'SmallIntegerField': _NUMERICAL_LOOKUP,
    'AutoField': _NUMERICAL_LOOKUP,
    # relateds
    'RelatedObject': _RELATED_LOOKUP,
    'ManyToManyField': _RELATED_LOOKUP,
    'ForeignKey': _RELATED_LOOKUP,
    'OneToOneField': _RELATED_LOOKUP,
    # chars
    'SlugField': _CHAR_LOOKUP,
    'CharField': _CHAR_LOOKUP,
    'TextField': _CHAR_LOOKUP,
    'EmailField': _CHAR_LOOKUP,
    'FileField': _CHAR_LOOKUP,
    'FilePathField': _CHAR_LOOKUP,
    'ImageField': _CHAR_LOOKUP,
    'IPAddressField': _CHAR_LOOKUP,
    'URLField': _CHAR_LOOKUP,
    'XMLField': _CHAR_LOOKUP,
    # datetime`s
    'DateTimeField': _DATETIME_LOOKUP,
    'DateField': _DATETIME_LOOKUP,
    'TimeField': _DATETIME_LOOKUP,
    # booleans
    'BooleanField': _BOOLEAN_LOOKUP,
}



from django.contrib.auth.decorators import (
    permission_required, login_required
)
from django.shortcuts import render
from queryset_reporter.models import Queryset
from queryset_reporter.core import Reporter
from queryset_reporter import __version__ as version


@login_required
@permission_required('queryset_reporter.can_use_reports')
def create(request):
    '''
    View to create report`s.
    '''
    def render_page():
        return render(request, 'queryset_reporter/create.html', context)

    # all the querysets
    querysets = Queryset.objects.all()
    context = {
        'querysets': querysets,
        'version': version,
    }

    try:
        qs = Queryset.objects.get(
            pk=request.GET.get('queryset')
        )
    except Queryset.DoesNotExist:
        return render_page()

    reporter = Reporter(qs, request)
    reporter.get_filters()

    file_format = request.GET.get('format')
    if file_format == 'csv':
        context.update({
            'rendered_csv': reporter.render_csv()
        })
    elif file_format == 'xlsx':
        context.update({
            'rendered_xlsx': reporter.render_xlsx()
        })

    context.update({
        'reporter': reporter,
        'request': request,
    })
    return render_page()



import re
import os
import uuid
from datetime import datetime

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.db.models import aggregates
from openpyxl import Workbook

from queryset_reporter.models import Queryset, QueryFilter


class Reporter(object):
    '''
    Manipulates a ``queryset_reporter.models.Queryset`` model to
    obtain another queryset based on ``Queryset`` metadada.
    '''

    def __init__(self, queryset, request=None):
        if isinstance(queryset, Queryset):
            self.queryset = queryset
            self.request = request
            self._request_filters()
            self._prepare_vars()
            self._filters()
        else:
            raise Exception(_('Instância de Modelo de Queryset inválida.'))

    def _request_filters(self):
        self.filters = []
        # get readonly filters
        for rof in self.queryset.queryfilter_set.filter(readonly=True):
            f = {'filter': rof}
            if rof.value_1:
                f.update({'values': [rof.value_0, rof.value_1]})
                self.filters.append(f)
            elif rof.value_0:
                f.update({'values': [rof.value_0]})
                self.filters.append(f)

        # returns the id`s of ``Filters`` objects of the request GET.
        if self.request:
            filter_ids = []
            _rfilter = re.compile(r'^filter-([\d]+)$')
            for x in self.request.GET:
                match = _rfilter.match(x)
                if match:
                    filter_ids.append(int(match.groups()[0]))

            # appends to self.filter a dict with key 'filter' equals the
            # ``Filter`` object and the values of the related filter.
            for fid in filter_ids:
                try:
                    _filter = self.queryset.queryfilter_set.get(id=fid)
                except QueryFilter.DoesNotExist:
                    continue
                if _filter not in [x['filter'] for x in self.filters]:
                    self.filters.append({
                        'filter': _filter,
                        'values': [
                            self.request.GET.get(v)
                            for v in sorted(self.request.GET)
                            if v.startswith('filter-%s-' % fid)
                        ]
                    })

    def _prepare_vars(self):
        # rsq -> result queryset
        model_class = self.queryset.model.model_class()
        self._rqs = model_class.objects.all()
        # fields -> display fields
        self.fields = self.queryset.displayfield_set.all()

    @staticmethod
    def _clean_values(values, config):
        '''Return a pythonic value of the given list of values
        '''
        def _clean_val(value, field_type):
            if field_type == 'datetime-field':
                return datetime.strptime(value, '%d/%m/%Y %H:%M')
            elif field_type == 'boolean-field':
                if value == '1':
                    return True
                else:
                    return False
            elif field_type == 'char-field':
                return value
            elif field_type == 'numerical-list':
                return [int(x.strip()) for x in value.split(',') if x.strip()]
            elif field_type == 'char-list':
                return [x.strip() for x in value.split(',') if x.strip()]
            elif field_type == 'numerical-field':
                if '.' in value:
                    return float(value)
                else:
                    return int(value)
            else:
                return None

        cleaned_values = [_clean_val(values[x], config[x][0])
                          for x in range(len(values))]
        if len(cleaned_values) == 1:
            return cleaned_values[0]
        else:
            return cleaned_values

    def _filters(self):
        for f in self.filters:
            flookup = '__'.join((f['filter'].field, f['filter'].lookup))
            fvalues = self._clean_values(f['values'], f['filter'].lookup_config)
            if f['filter'].method == u'filter':
                # append filter method
                self._rqs = self._rqs.filter(**{flookup: fvalues})
            elif f['filter'].method == u'exclude':
                # append exclude method
                self._rqs = self._rqs.exclude(**{flookup: fvalues})
        if self.queryset.distinct:
            self._rqs = self._rqs.distinct()

    def _fields_list(self):
        fields_list = []
        for f in self.fields:
            if not f.annotate:
                fields_list.append(f.field)
        return fields_list

    def _order_by(self):
        # f-> field, s -> sort
        # if sort is 'desc' prepend a `-` in front of `f` string
        sort_field = lambda f, s: (u'-%s' % f) if s == u'desc' else f
        # order_by <- a list of order_by fields with order signals
        # i.e: ['title', '-creation_time']
        return [
            sort_field((
                x.field if not x.annotate
                else ('%s_%s' % (x.annotate, x.field)).replace('__', '_')
            ), x.sort) for x in
            self.queryset.displayfield_set.filter(sort__isnull=False)
        ]

    def _annotate_dict(self):
        annot_list = []
        for x in self.queryset.displayfield_set.filter(annotate__isnull=False):
            key = ('%s_%s' % (x.annotate, x.field)).replace('__', '_')
            try:
                agg_class = getattr(aggregates, x.annotate)
            except AttributeError:
                continue
            else:
                annot_list.append([key, agg_class(x.field)])
        return dict(annot_list)

    def get_base_qs(self):
        order = self._order_by()
        fields = self._fields_list()
        annot = self._annotate_dict()
        if annot:
            return self._rqs.values(*fields).annotate(**annot).order_by(*order)
        else:
            return self._rqs.values(*fields).order_by(*order)

    def _get_queryfilters(self, method):
        '''
        Returns a list of ``queryfilter``s of the queryset with the kind of
        the given ``method`` (possible values is `filter` or `exclude`).
        '''
        if method not in ('filter', 'exclude'):
            raise Exception(
                'Invalid ``method`` argument, must be `filter` or `exclude`.'
            )
        filter_dict = dict([
            (x['filter'], x['values'])
            for x in self.filters if x['filter'].method == method
        ])
        return [
            (x, x.lookup_config, filter_dict.get(x))
            for x in self.queryset.queryfilter_set.filter(method=method)
        ]

    def get_filters(self):
        return self._get_queryfilters('filter')

    def get_excludes(self):
        return self._get_queryfilters('exclude')

    def preview(self, limit=50):
        return self.get_base_qs()[:limit]

    def count(self):
        return self.get_base_qs().count()

    def render_xlsx(self):
        wb = Workbook()
        ws = wb.create_sheet()

        ws.append(
            [x.field_verbose for x in self.queryset.displayfield_set.all()])
        for line in self.get_base_qs():
            _list = []
            for field in self.fields:
                pre_concat = (
                    field.pre_concatenate if field.pre_concatenate else ''
                )
                pos_concat = (
                    field.pos_concatenate if field.pos_concatenate else ''
                )
                line_value = line.get(field.get_field)
                _list.append(f'{pre_concat}{line_value}{pos_concat}')
            ws.append(_list)
        file_name = 'xlsx/%s.xlsx' % uuid.uuid4().hex
        file_path = os.path.join(settings.MEDIA_ROOT, file_name)
        wb.save(file_path)

        return settings.MEDIA_URL + file_name



from django.contrib import admin
from queryset_reporter.models import Queryset, QueryFilter, DisplayField
from queryset_reporter.forms import DisplayFieldForm, QueryFilterForm


class DisplayFieldInline(admin.TabularInline):
    model = DisplayField
    form = DisplayFieldForm
    extra = 0
    classes = ('collapse open',)
    inline_classes = ('collapse',)
    sortable_field_name = "position"
    fields = (
        'field', 'field_verbose', 'field_type', 'model_field', 'sort',
        'annotate', 'position', 'pre_concatenate', 'pos_concatenate'
    )


class QueryFilterInline(admin.TabularInline):
    model = QueryFilter
    form = QueryFilterForm
    extra = 0
    classes = ('collapse open',)
    inline_classes = ('collapse',)


class QuerysetAdmin(admin.ModelAdmin):
    change_form_template = 'admin/queryset_reporter_change_form.html'
    list_display = ('name', 'model', 'distinct', 'created_at', 'modified_at',
        'automatic_generation', 'last_automatic_generation_at')
    list_filter = ('model', 'created_at', 'modified_at', 'distinct',
        'automatic_generation')
    inlines = (DisplayFieldInline, QueryFilterInline)
    fields = ('name', 'desc', 'model', 'distinct', 'automatic_generation')

    class Media:
        js = (
            "queryset_reporter/admin.js",
            "https://code.jquery.com/jquery-1.4.4.min.js"
        )
        css = {
            'all': ("queryset_reporter/admin.css",)
        }

admin.site.register(Queryset, QuerysetAdmin)



# Generated by Django 2.2.5 on 2019-09-27 04:44

from django.db import migrations, models
import django.db.models.deletion
import queryset_reporter.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Queryset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Nome')),
                ('desc', models.TextField(blank=True, max_length=255, null=True, verbose_name='Descrição')),
                ('distinct', models.BooleanField(default=False, help_text='\n        Útil quando relatórios que acessam muitas tabelas tem a possibilidade\n        de retornar resultados repetidos, marcar este campo desabilita a\n        repetição.', verbose_name='Distinguir')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Criação')),
                ('modified_at', models.DateTimeField(auto_now=True, verbose_name='Modificação')),
                ('automatic_generation', models.BooleanField(default=False, verbose_name='Geração Automática')),
                ('last_automatic_generation_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Última geração automática')),
                ('last_automatic_generation_xlsx', models.CharField(blank=True, editable=False, max_length=250, null=True, verbose_name='Último relatório gerado em XLSX')),
                ('model', models.ForeignKey(limit_choices_to={'pk__in': queryset_reporter.models.Queryset._get_allowed_models}, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Modelo')),
            ],
            options={
                'verbose_name': 'Modelo de Relatório',
                'verbose_name_plural': 'Modelos de Relatórios',
            },
        ),
        migrations.CreateModel(
            name='QueryFilter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.CharField(blank=True, max_length=255, null=True, verbose_name='Código do Campo')),
                ('field_verbose', models.CharField(blank=True, max_length=255, null=True, verbose_name='Nome do Campo')),
                ('field_type', models.CharField(blank=True, max_length=255, null=True, verbose_name='Tipo do Campo')),
                ('lookup', models.CharField(choices=[('exact', 'Termo exato'), ('iexact', 'Termo exato (case-insensitivo)'), ('contains', 'Contém o termo'), ('icontains', 'Contém o termo (case-insensitivo)'), ('in', 'Termo está na lista'), ('gt', 'Maior que'), ('gte', 'Maior ou igual que'), ('lt', 'Menor que'), ('lte', 'Menor ou igual que'), ('startswith', 'Começa com'), ('istartswith', 'Começa com (case-insensitivo)'), ('endswith', 'Termina com'), ('iendswith', 'Termina com (case-insensitivo)'), ('range', 'Faixa/período'), ('year', 'Ano específico'), ('month', 'Mês específico'), ('day', 'Dia específico.'), ('isnull', 'É nulo?'), ('search', 'Busca textual'), ('regex', 'Expressão regular'), ('iregex', 'Expressão regular (case-insensitivo)')], max_length=11, verbose_name='Tipo de filtro')),
                ('method', models.CharField(choices=[('filter', 'Filtro'), ('exclude', 'Exclusão')], editable=False, max_length=7, verbose_name='Método')),
                ('readonly', models.BooleanField(default=False)),
                ('value_0', models.CharField(blank=True, max_length=255, null=True, verbose_name='Valor padrão 1')),
                ('value_1', models.CharField(blank=True, max_length=255, null=True, verbose_name='Valor padrão 2')),
                ('queryset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='queryset_reporter.Queryset')),
            ],
            options={
                'verbose_name': 'Filtros de query`s',
            },
        ),
        migrations.CreateModel(
            name='DisplayField',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.CharField(blank=True, max_length=255, null=True, verbose_name='Código do Campo')),
                ('field_verbose', models.CharField(blank=True, max_length=255, null=True, verbose_name='Nome do Campo')),
                ('field_type', models.CharField(blank=True, max_length=255, null=True, verbose_name='Tipo do Campo')),
                ('sort', models.CharField(blank=True, choices=[('asc', 'Crescente'), ('desc', 'Decrescente')], default=None, max_length=4, null=True, verbose_name='Ordenação')),
                ('annotate', models.CharField(blank=True, choices=[('Count', 'Somatório'), ('Ave', 'Média'), ('Max', 'Máximo'), ('Min', 'Mínimo')], max_length=5, null=True, verbose_name='Anotação')),
                ('position', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('pre_concatenate', models.CharField(blank=True, max_length=255, null=True, verbose_name='Pré concatenação')),
                ('pos_concatenate', models.CharField(blank=True, max_length=255, null=True, verbose_name='Pós concatenação')),
                ('queryset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='queryset_reporter.Queryset')),
            ],
            options={
                'verbose_name': 'Campo a exibir',
                'verbose_name_plural': 'Campos à exibir',
                'ordering': ['position'],
            },
        ),
        migrations.CreateModel(
            name='Exclude',
            fields=[
            ],
            options={
                'verbose_name': 'Exclusão',
                'verbose_name_plural': 'Exclusões',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('queryset_reporter.queryfilter',),
        ),
        migrations.CreateModel(
            name='Filter',
            fields=[
            ],
            options={
                'verbose_name': 'Filtro',
                'verbose_name_plural': 'Filtros',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('queryset_reporter.queryfilter',),
        ),
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _

from queryset_reporter.models import DisplayField, QueryFilter
from queryset_reporter import settings


def get_widget_input(attrs):
    '''Returns a input widget for the "magic fields" of FieldModel
    based on project settings.
    '''
    if settings.QR_HIDE_FIELDS:
        return forms.HiddenInput(attrs=attrs)
    else:
        return forms.TextInput(attrs=attrs)


class FieldModelForm(forms.ModelForm):
    '''Customizes `field`, `field_verbose` and `field_type`; adds `model_type`
    '''

    field = forms.CharField(
        label=_(u'Nome do campo'), required=True,
        widget=get_widget_input(
            {'class': 'introspect-field', 'readonly': 'readonly'}
        )
    )

    field_verbose = forms.CharField(
        label=_(u'Apelido do campo'), required=True,
        widget=get_widget_input(
            {'class': 'introspect-field_verbose'}
        )
    )

    field_type = forms.CharField(
        label=_(u'Tipo do campo'), required=True,
        widget=get_widget_input(
            {'class': 'introspect-field_type', 'readonly': 'readonly'}
        )
    )

    model_field = forms.CharField(
        label=_(u'Buscar campo'), required=False,
        widget=forms.Select(
            attrs={'class': 'introspect-model_field'}
        )
    )


class DisplayFieldForm(FieldModelForm):
    position = forms.IntegerField(required=False, widget=forms.HiddenInput(
        attrs={'class': 'field'}
    ))

    class Meta:
        model = DisplayField
        fields = [
            'queryset', 'field', 'field_verbose', 'field_type', 'model_field',
            'sort', 'annotate', 'position', 'pre_concatenate', 'pos_concatenate'
        ]


class QueryFilterForm(FieldModelForm):
    class Meta:
        model = QueryFilter
        fields = [
            'queryset', 'field', 'field_verbose', 'field_type', 'model_field',
            'method', 'lookup', 'readonly', 'value_0', 'value_1'
        ]



'''
Models of queryset_reporter.
'''

from django.utils.translation import ugettext_lazy as _
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse

from queryset_reporter import mapping
from queryset_reporter import settings

_NULL = {'null': True, 'blank': True}
_CHAR = {'max_length': 255, 'blank': False}
_CNULL = _CHAR
_CNULL.update(_NULL)


class Queryset(models.Model):
    '''Queryset is a model representation of a generic Model.
    '''

    def _get_allowed_models():
        '''Return allowed models based on project settings.
        '''
        models = ContentType.objects.all()
        if settings.QUERYSET_REPORTER_INCLUDE_APPS:
            models = models.filter(
                app_label__in=settings.QUERYSET_REPORTER_INCLUDE_APPS
            )
        if settings.QUERYSET_REPORTER_EXCLUDE_APPS:
            models = models.exclude(
                app_label__in=settings.QUERYSET_REPORTER_EXCLUDE_APPS
            )
        return list(models.values_list('pk', flat=True))

    # metadata
    name = models.CharField(_(u'Nome'), **_CHAR)
    desc = models.TextField(_(u'Descrição'), **_CHAR)
    model = models.ForeignKey(
        ContentType, verbose_name=_(u'Modelo'),
        limit_choices_to={'pk__in': _get_allowed_models()},
        on_delete=models.CASCADE
    )
    distinct = models.BooleanField(_(u'Distinguir'), default=False)

    # timestamps
    created_at = models.DateTimeField(_(u'Criação'), auto_now_add=True)
    modified_at = models.DateTimeField(_(u'Modificação'), auto_now=True)

    # automatic generation
    automatic_generation = models.BooleanField(
        _(u'Geração Automática'), default=False)
    last_automatic_generation_at = models.DateTimeField(
        _(u'Última geração automática'), editable=False, **_NULL)
    last_automatic_generation_xlsx = models.CharField(
        _(u'Último relatório gerado em XLSX'), max_length=250,
        editable=False, **_NULL)

    def __str__(self):
        return f'[{self.model.name}] {self.name}'

    class Meta:
        verbose_name = _(u'Modelo de Queryset')
        verbose_name_plural = _(u'Modelos de Queryset')
        permissions = (
            ('can_use_reports', 'Prever e gerar relatórios'),
        )

    def get_absolute_url(self):
        path = reverse('qr_create')
        return f'{path}?queryset={self.id}'


class FieldModel(models.Model):
    queryset = models.ForeignKey(Queryset, on_delete=models.CASCADE)
    field = models.CharField(_(u'Código do Campo'), **_CHAR)
    field_verbose = models.CharField(_(u'Nome do Campo'), **_CHAR)
    field_type = models.CharField(_(u'Tipo do Campo'), **_CHAR)

    class Meta:
        abstract = True


class DisplayField(FieldModel):
    '''Or the Fields and Extras selects called in .values().
    '''

    sort = models.CharField(
        verbose_name=_(u'Ordenação'), max_length=4, choices=(
            ('asc', _(u'Crescente')),
            ('desc', _(u'Decrescente'))
        ), default=None, **_NULL
    )
    annotate = models.CharField(
        verbose_name=_(u'Anotação'), max_length=5, choices=(
            ('Count', _(u'Somatório')),
            ('Ave', _(u'Média')),
            ('Max', _(u'Máximo')),
            ('Min', _(u'Mínimo')),
        ), **_NULL
    )
    position = models.PositiveSmallIntegerField(**_NULL)

    pre_concatenate = models.CharField(
        verbose_name=_(u'Pré concatenação'), **_CNULL
    )
    pos_concatenate = models.CharField(
        verbose_name=_(u'Pós concatenação'), **_CNULL
    )

    def __str__(self):
        return self.field_verbose

    @property
    def get_field(self):
        if not self.annotate:
            return self.field
        else:
            return '%s_%s' % (self.annotate, self.field.replace('__', '_'))

    class Meta:
        verbose_name = _(u'Campo a exibir')
        verbose_name_plural = _(u'Campos à exibir')
        ordering = ['position']


class QueryFilter(FieldModel):
    '''
    QueryFilter of a Queryset
    '''

    LOOKUPS = mapping.LOOKUPS
    FILTER_METHODS = (
        ('filter', _(u'Filtro')),
        ('exclude', _(u'Exclusão')),
    )

    lookup = models.CharField(
        _(u'Tipo de filtro'), max_length=max([len(x[0]) for x in LOOKUPS]),
        choices=LOOKUPS)
    method = models.CharField(
        _(u'Método'), choices=FILTER_METHODS,
        max_length=max([len(x[0]) for x in FILTER_METHODS])
    )
    readonly = models.BooleanField(default=False)
    value_0 = models.CharField(_(u'Valor padrão 1'), **_CNULL)
    value_1 = models.CharField(_(u'Valor padrão 2'), **_CNULL)

    def __str__(self):
        return '%s por %s' % (self.field_verbose, self.get_lookup_display())

    @property
    def lookup_config(self):
        lookup_info = mapping.LOOKUP_MAPPING[self.field_type]
        if self.lookup:
            lookup_config = lookup_info.get(self.lookup)
        else:
            lookup_config = None
        # returns something like
        # [('numerical-field', 0), ('numerical-field', 1)] or None
        return [
            (lookup_config[1], x) for x in range(lookup_config[0])
        ] if lookup_config else None

    class Meta:
        verbose_name = _(u'Filtro de query')
        verbose_name_plural = _(u'Filtros de query`s')



# Generated by Django 2.2.5 on 2019-10-03 01:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('queryset_reporter', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Exclude',
        ),
        migrations.DeleteModel(
            name='Filter',
        ),
        migrations.AlterModelOptions(
            name='queryfilter',
            options={'verbose_name': 'Filtro de query', 'verbose_name_plural': 'Filtros de query`s'},
        ),
        migrations.AlterModelOptions(
            name='queryset',
            options={'verbose_name': 'Modelo de Queryset', 'verbose_name_plural': 'Modelos de Queryset'},
        ),
        migrations.AlterField(
            model_name='queryfilter',
            name='method',
            field=models.CharField(choices=[('filter', 'Filtro'), ('exclude', 'Exclusão')], max_length=7, verbose_name='Método'),
        ),
        migrations.AlterField(
            model_name='queryset',
            name='distinct',
            field=models.BooleanField(default=False, verbose_name='Distinguir'),
        ),
        migrations.AlterField(
            model_name='queryset',
            name='model',
            field=models.ForeignKey(limit_choices_to={'pk__in': [2, 3, 4, 7, 8, 9, 10]}, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Modelo'),
        ),
    ]



"""
queryset_reporter tests.
"""

from django.test import TestCase


class AjaxViewsTest(TestCase):
    def test_ajax_return_aways_200(self):
        '''
        Test in a range of 0 to 100, if all the ajax calls to model:
        /queryset_reporter/ajax/model-fields/?model=<id> returns a 200 code.
        '''
        for i in range(100):
            url = '/queryset_reporter/ajax/model-fields/?model=%d'
            response = self.client.get(url % i)
            self.assertUnlessEqual(response.status_code, 200)

    def test_ajax_return_with_none_model_id(self):
        '''
        Test if the returns is 200 if the model id is none.
        '''
        response = self.client.get('/queryset_reporter/ajax/model-fields/?model=')
        self.assertUnlessEqual(response.status_code, 200)



from django.conf import settings
from django.db.models.fields import Field
from django.db.models.fields.reverse_related import ManyToOneRel, ManyToManyRel
from django.db.models.fields.related import ManyToManyField, ForeignKey

_DEFAULT_RECURSION_DEPTH = 2
_MAX_RECURSION_DEPTH = getattr(
    settings, 'QUERYSET_REPORTER_MAX_RECURSION_DEPTH', _DEFAULT_RECURSION_DEPTH
)


def get_class_name(object):
    return object.__class__.__name__


def _direct_field(field, recursion_depth=0):
    field_metadata = {
        'name': field.name,
        'verbose': field.verbose_name.format('%s'),
        'type': get_class_name(field)
    }
    if isinstance(field, (ManyToManyField, ForeignKey)):
        if recursion_depth > _MAX_RECURSION_DEPTH:
            return None

        field_metadata.update({
            'lookup_fields': get_model_fields(
                field.target_field.model, recursion_depth=recursion_depth
            )
        })
    return field_metadata


def _related_object(field, recursion_depth=0):
    related_object = {
        'name': field.name,
        'verbose': field.name.capitalize(),
        'type': get_class_name(field),
    }
    if recursion_depth <= _MAX_RECURSION_DEPTH:
        related_object.update({
            'lookup_fields': get_model_fields(
                field.field.model, recursion_depth=recursion_depth
            )
        })
    return related_object


def _field_meta(field, recursion_depth=0):
    '''
    Return the name of the field, your verbose name, your type and your lookup
    fields, if exist.
    '''

    if isinstance(field, (ManyToOneRel, ManyToManyRel)):
        return _related_object(field, recursion_depth)
    elif isinstance(field, Field):
        return _direct_field(field, recursion_depth)


def get_model_fields(model, recursion_depth=0):
    '''
    Make a instrospection in the fields of an given model and return all the
    fields with attribute name, verbose name and type.
    For related fields, like ForeignKey, ManyToManyField, OneToOneField and
    not direct fields, return them into key `lookup_fields`, and, for each of
    them, call this function again, respecting the `_MAX_RECURSION_DEPTH` conf.
    '''

    fields = []
    recursion_depth += 1

    for field in model._meta.get_fields():
        f = _field_meta(field, recursion_depth)
        if f:
            fields.append(f)
    return fields



from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.decorators import permission_required
from django.http import JsonResponse
from django.utils.translation import ugettext as _
from queryset_reporter.introspect import get_model_fields


@permission_required('is_staff')
def model_fields(request):
    '''Returns a list of dicts that`s represents the fields metadata of an
    given model.
    '''

    # get`s the model from request.GET and return a error if doesn`t exist
    try:
        ctype = ContentType.objects.get(pk=request.GET.get('model'))
    except ContentType.DoesNotExist:
        return JsonResponse({
            'success': False, 'error': _(u'Modelo inexiste.')})
    except ValueError:
        return JsonResponse({
            'success': False, 'error': _(u'Identificador do modelo inválido')})

    model = ctype.model_class()
    if not model:
        return JsonResponse({
            'success': False, 'error': _(u'Modelo abstrato.')
        })

    return JsonResponse({
        'success': True,
        'model': str(model),
        'model_verbose': model._meta.verbose_name.format('%s'),
        'fields': get_model_fields(model),
    })



# Generated by Django 2.2.6 on 2019-10-29 18:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('queryset_reporter', '0002_auto_20191003_0105'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='queryset',
            options={'permissions': (('can_use_reports', 'Prever e gerar relatórios'),), 'verbose_name': 'Modelo de Queryset', 'verbose_name_plural': 'Modelos de Queryset'},
        ),
    ]



from django.urls import path, reverse
from django.views.generic.base import RedirectView
from queryset_reporter.views.ajax import model_fields
from queryset_reporter.views.core import create


app_name = 'queryset_reporter'
urlpatterns = [
    path(
        'api/model-fields', model_fields, name='qr_api_modelfields'
    ),
    path(
        'create', create, name='qr_create'
    ),
    path(
        '', RedirectView.as_view(pattern_name='queryset_reporter:qr_create'),
        name='qr_index'
    ),
]



import pathlib
from queryset_reporter import __version__
from setuptools import setup, find_packages


HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name='queryset_reporter',
    version=__version__,
    url='http://github.com/ricardodani/django-queryset-reporter',
    author='Ricardo Dani',
    author_email='ricardodani@gmail.com',
    description=u'App that creates models of Queryset and generate reports',
    long_description=README,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=['openpyxl'],
    packages=find_packages(),
    python_requires=">3.5",
    license="MIT",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



from django.conf import settings as django_settings

'''
Defines the `app_labels` that will be excluded from model selection at
`Queryset` form. Empty list means no excluding.
'''
_QUERYSET_REPORTER_EXCLUDE_APPS_DEFAULT_VALUE = [
    'admin', 'contenttypes', 'sessions', 'queryset_reporter'
]
QUERYSET_REPORTER_EXCLUDE_APPS = getattr(
    django_settings,
    'QUERYSET_REPORTER_EXCLUDE_APPS',
    _QUERYSET_REPORTER_EXCLUDE_APPS_DEFAULT_VALUE
)

'''
Defines the `app_labels` that will be included exclusively in model field at
`Queryset` form. Emtpy list means no filtering.
'''
_QUERYSET_REPORTER_INCLUDE_APPS_DEFAULT_VALUE = []
QUERYSET_REPORTER_INCLUDE_APPS = getattr(
    django_settings,
    'QUERYSET_REPORTER_INCLUDE_APPS',
    _QUERYSET_REPORTER_INCLUDE_APPS_DEFAULT_VALUE
)


'''
Defaults to True, if False, `field`, `field_verbose`, `field_type` are
displayed in admin.
'''
QR_HIDE_FIELDS = getattr(
    django_settings, 'QR_HIDE_FIELDS', True
)
